import { compact } from '../../util/array.js';
import { dirname, isInternal, toAbsolute } from '../../util/path.js';
import { timerify } from '../../util/Performance.js';
import { hasDependency, load } from '../../util/plugin.js';
export const NAME = 'TypeScript';
export const ENABLERS = ['typescript'];
export const isEnabled = ({ dependencies }) => hasDependency(dependencies, ENABLERS);
export const CONFIG_FILE_PATTERNS = ['tsconfig.json'];
const resolveExtensibleConfig = async (configFilePath) => {
    const config = await load(configFilePath);
    if (config?.extends) {
        for (const extend of [config.extends].flat()) {
            if (isInternal(extend)) {
                const presetConfigPath = toAbsolute(extend, dirname(configFilePath));
                const presetConfig = await resolveExtensibleConfig(presetConfigPath);
                Object.assign(config, presetConfig);
            }
        }
    }
    return config;
};
const findTypeScriptDependencies = async (configFilePath) => {
    const config = await resolveExtensibleConfig(configFilePath);
    if (!config)
        return [];
    const extend = config.extends ? [config.extends].flat().filter(extend => !isInternal(extend)) : [];
    const plugins = compact(config.compilerOptions?.plugins?.map(plugin => plugin.name) ?? []);
    const importHelpers = config.compilerOptions?.importHelpers ? ['tslib'] : [];
    const jsx = config.compilerOptions?.jsxImportSource
        ? [config.compilerOptions.jsxImportSource]
        : config.compilerOptions?.jsx
            ? ['react']
            : [];
    return [...extend, ...plugins, ...importHelpers, ...jsx];
};
export const findDependencies = timerify(findTypeScriptDependencies);
